PrefabFiles = 
{
    "largechest",
    "largeicebox",
}

Assets = 
{
    Asset("ATLAS", "images/inventoryimages/largechest.xml"),
    Asset("IMAGE", "images/inventoryimages/largechest.tex"),
    Asset("ANIM", "anim/ui_largechest_5x5.zip"),
}

local require = GLOBAL.require
STRINGS = GLOBAL.STRINGS
RECIPETABS = GLOBAL.RECIPETABS
Recipe = GLOBAL.Recipe
Ingredient = GLOBAL.Ingredient
TECH = GLOBAL.TECH

local containers = require "containers"

local params = {}

local OVERRIDE_WIDGETSETUP = false
local containers_widgetsetup_base = containers.widgetsetup

function containers.widgetsetup(container, prefab)
    local t = params[prefab or container.inst.prefab]
    if t ~= nil then
        for k, v in pairs(t) do
            container[k] = v
        end
        container:SetNumSlots(container.widget.slotpos ~= nil and #container.widget.slotpos or 0)
        if OVERRIDE_WIDGETSETUP then
            container.type = "largechest"
        end
    else
        containers_widgetsetup_base(container, prefab)
    end
end

local function makeChest()
    local container =
    {
        widget =
        {
            slotpos = {},
            animbank = "ui_largechest_5x5",
            animbuild = "ui_largechest_5x5",
            pos = GLOBAL.Vector3(0, 200, 0),
            side_align_tip = 160,
        },
        type = "chest",
    }

    for y = 3, -1, -1 do
        for x = -1, 3 do
            table.insert(container.widget.slotpos, GLOBAL.Vector3(80 * x - 80 * 2 + 80, 80 * y - 80 * 2 + 80, 0))
        end
    end

    return container
end

--------------------------------------------------------------------------
--[[ largechest ]]
--------------------------------------------------------------------------

params.largechest = makeChest()

--------------------------------------------------------------------------
--[[ largechest ]]
--------------------------------------------------------------------------

params.largeicebox = makeChest()

function params.largeicebox.itemtestfn(container, item, slot)
    if item.prefab == "spoiled_food" then
        return true
    end

    --Perishable
    if not (item:HasTag("fresh") or item:HasTag("stale") or item:HasTag("spoiled")) then
        return false
    end

    --Edible
    for k, v in pairs(GLOBAL.FOODTYPE) do
        if item:HasTag("edible_"..v) then
            return true
        end
    end

    return false
end
--------------------------------------------------------------------------

for k, v in pairs(params) do
    containers.MAXITEMSLOTS = math.max(containers.MAXITEMSLOTS, v.widget.slotpos ~= nil and #v.widget.slotpos or 0)
end

--------------------------------------------------------------------------


GLOBAL.LARGECHEST_OPT_DIFFICULTY = GetModConfigData("OPT_DIFFICULTY")

local recipe1 = Recipe("largechest", { Ingredient("boards", 8), Ingredient("goldnugget", 2) }, RECIPETABS.TOWN, TECH.SCIENCE_ONE, "largechest_placer")
recipe1.atlas = "images/inventoryimages/largechest.xml"

local recipe2 = Recipe("largeicebox", { Ingredient("goldnugget", 6), Ingredient("gears", 3), Ingredient("boards", 3) }, RECIPETABS.FARM,  TECH.SCIENCE_TWO, "largeicebox_placer")
recipe2.image = "icebox.tex"

local function update_recipe(self, ingredients, tab, level, placer, min_spacing, nounlock, numtogive)
    self.ingredients   = ingredients
    self.tab           = tab
    self.level         = level or 0
    self.level.ANCIENT = self.level.ANCIENT or 0
    self.level.MAGIC   = self.level.MAGIC or 0
    self.level.SCIENCE = self.level.SCIENCE or 0
    self.placer        = placer
    self.min_spacing   = min_spacing or 3.2
    self.nounlock      = nounlock or false
    self.numtogive     = numtogive or 1
end

local function register_recipes()
    if GLOBAL.LARGECHEST_OPT_DIFFICULTY == 2 then
        update_recipe(recipe1, { Ingredient("boards", 8), Ingredient("goldnugget", 4) }, RECIPETABS.TOWN, TECH.SCIENCE_TWO, "largechest_placer")
    elseif GLOBAL.LARGECHEST_OPT_DIFFICULTY == 1 then
        update_recipe(recipe1, { Ingredient("boards", 8), Ingredient("goldnugget", 2) }, RECIPETABS.TOWN, TECH.SCIENCE_TWO, "largechest_placer")
    else
        update_recipe(recipe1, { Ingredient("boards", 6) }, RECIPETABS.TOWN, TECH.SCIENCE_ONE, "largechest_placer")
    end
    if GLOBAL.LARGECHEST_OPT_DIFFICULTY == 2 then
        update_recipe(recipe2, { Ingredient("goldnugget", 8), Ingredient("gears", 3), Ingredient("boards", 4) }, RECIPETABS.FARM,  TECH.SCIENCE_TWO, "largeicebox_placer")
    elseif GLOBAL.LARGECHEST_OPT_DIFFICULTY == 1 then
        update_recipe(recipe2, { Ingredient("goldnugget", 6), Ingredient("gears", 3), Ingredient("boards", 3) }, RECIPETABS.FARM,  TECH.SCIENCE_TWO, "largeicebox_placer")
    else
        update_recipe(recipe2, { Ingredient("goldnugget", 4), Ingredient("gears", 2), Ingredient("boards", 2) }, RECIPETABS.FARM,  TECH.SCIENCE_TWO, "largeicebox_placer")
    end
end

register_recipes()

GLOBAL.LARGECHEST_REGISTER_RECIPES = register_recipes
GLOBAL.LARGECHEST_CONFIG_SYNC = false

STRINGS.NAMES.LARGECHEST = "Large Chest"
STRINGS.RECIPE_DESC.LARGECHEST = "A large chest to put more stuff"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.LARGECHEST = "Looks so fancy!"

STRINGS.NAMES.LARGEICEBOX = "Large Icebox"
STRINGS.RECIPE_DESC.LARGEICEBOX = "A gigantic icebox."
STRINGS.CHARACTERS.GENERIC.DESCRIBE.LARGEICEBOX = "I have harnessed the power of cold!"

AddPrefabPostInit("world_network", function(inst)
    inst:AddComponent("largechest_configsync")
end)

--------------------------------------------------------------------------

local containers_widgetsetup_custom = containers.widgetsetup
local MAXITEMSLOTS = containers.MAXITEMSLOTS

AddPrefabPostInit("world_network", function(inst)
    if containers.widgetsetup ~= containers_widgetsetup_custom then
        OVERRIDE_WIDGETSETUP = true
        local containers_widgetsetup_base2 = containers.widgetsetup
        function containers.widgetsetup(container, prefab)
            containers_widgetsetup_base2(container, prefab)
            if container.type == "largechest" then
                container.type = "chest"
            end
        end
    end
    if containers.MAXITEMSLOTS < MAXITEMSLOTS then
        containers.MAXITEMSLOTS = MAXITEMSLOTS
    end
end)
